﻿#include "CMGameInstance.h"
#include "SocketIOClient.h"

void UCMGameInstance::Login(const FString& Username, const FString& Password)
{
	const auto Request = FHttpModule::Get().CreateRequest();
	Request->SetURL(LobbyAPI + "/login");
	Request->SetVerb("POST");
	Request->SetHeader("Content-Type", "application/json");
	Request->SetHeader("Accept", "application/json");
	Request->SetContentAsString("{\"username\":\"" + Username + "\",\"password\":\"" + Password + "\"}");
	Request->OnProcessRequestComplete().BindUObject(this, &UCMGameInstance::OnRequestToLoginComplete);
	Request->ProcessRequest();
}

void UCMGameInstance::Register(const FString& Username, const FString& Password)
{
	const auto Request = FHttpModule::Get().CreateRequest();
	Request->SetURL(LobbyAPI + "/register");
	Request->SetVerb("POST");
	Request->SetHeader("Content-Type", "application/json");
	Request->SetHeader("Accept", "application/json");
	Request->SetContentAsString("{\"username\":\"" + Username + "\",\"password\":\"" + Password + "\"}");
	Request->OnProcessRequestComplete().BindUObject(this, &UCMGameInstance::OnRequestToRegisterComplete);
	Request->ProcessRequest();
}

void UCMGameInstance::Logout()
{
	const auto Request = FHttpModule::Get().CreateRequest();
	Request->SetURL(LobbyAPI + "/logout");
	Request->SetVerb("POST");
	Request->SetHeader("Authorization", "Bearer " + JwtToken);
	Request->OnProcessRequestComplete().BindUObject(this, &UCMGameInstance::OnRequestToLogoutComplete);
	Request->ProcessRequest();
}

void UCMGameInstance::CreateLobby()
{
	const auto Request = FHttpModule::Get().CreateRequest();
	Request->SetURL(LobbyAPI + "/create");
	Request->SetVerb("POST");
	Request->SetHeader("Authorization", "Bearer " + JwtToken);
	Request->SetHeader("Content-Type", "application/json");
	Request->SetHeader("Accept", "application/json");
	Request->SetContentAsString("{\"maxPlayers\":\"4\"}");
	Request->OnProcessRequestComplete().BindUObject(this, &UCMGameInstance::OnRequestToCreateLobbyComplete);
	Request->ProcessRequest();
}

void UCMGameInstance::JoinLobby()
{
	// if (APlayerController* PC = GetWorld()->GetFirstPlayerController())
	// {
	// 	PC->ConsoleCommand("open " + ServerFQDN + ":" + FString::FromInt(Port));
	// }

	const auto Request = FHttpModule::Get().CreateRequest();
	Request->SetURL("/join");
	Request->SetVerb("POST");
	Request->SetHeader("Authorization", "Bearer " + JwtToken);
	Request->SetHeader("Content-Type", "application/json");
	Request->SetHeader("Accept", "application/json");
	const TSharedPtr<FJsonObject> JoinData = MakeShareable(new FJsonObject);
	JoinData->SetStringField("lobbyID", LobbyID);
	JoinData->SetStringField("username", AccountUsername);
	Request->SetContentAsString(USIOJConvert::ToJsonString(JoinData));
	Request->OnProcessRequestComplete().BindUObject(this, &UCMGameInstance::OnRequestToJoinLobbyComplete);
	Request->ProcessRequest();
}

void UCMGameInstance::LeaveLobby()
{
	const auto Request = FHttpModule::Get().CreateRequest();
	Request->SetURL("/leave");
	Request->SetVerb("POST");
	Request->SetHeader("Authorization", "Bearer " + JwtToken);
	Request->SetHeader("Accept", "application/json");
	Request->SetHeader("Accept", "application/json");
	const TSharedPtr<FJsonObject> JoinData = MakeShareable(new FJsonObject);
	JoinData->SetStringField("lobbyID", LobbyID);
	JoinData->SetStringField("username", AccountUsername);
	Request->SetContentAsString(USIOJConvert::ToJsonString(JoinData));
	Request->OnProcessRequestComplete().BindUObject(this, &UCMGameInstance::OnRequestToLeaveLobbyComplete);
	Request->ProcessRequest();
}

void UCMGameInstance::Init()
{
	Super::Init();

	Socket = ISocketIOClientModule::Get().NewValidNativePointer();

	Socket->Connect("wss://cm.api.philipwhite.dev/");
	Socket->OnConnectedCallback = [this](const FString& SocketID, const FString& SessionID)
	{
		UE_LOG(LogTemp, Log, TEXT("Connected: %s %s"), *SocketID, *SessionID);
		if (IsDedicatedServerInstance())
		{
			FParse::Value(FCommandLine::Get(), TEXT("lobbyID="), LobbyID);
			FString CMServerSecret;
			FParse::Value(FCommandLine::Get(), TEXT("CMServerSecret="), CMServerSecret);
			const TSharedPtr<FJsonObject> AuthorityData = MakeShareable(new FJsonObject);
			AuthorityData->SetStringField("lobbyID", LobbyID);
			AuthorityData->SetStringField("secret", CMServerSecret);
			Socket->Emit(TEXT("authority"), AuthorityData);
		}
		else
		{
			const TSharedPtr<FJsonObject> AuthorityData = MakeShareable(new FJsonObject);
			AuthorityData->SetStringField("authority", "client");
			Socket->Emit(TEXT("authority"), AuthorityData);
		}
	};

	Socket->OnEvent("command", [this](const FString& Event, const TSharedPtr<FJsonValue>& Command)
	{
		if (IsDedicatedServerInstance())
		{
			if (Command->AsString() == "stop")
			{
				UE_LOG(LogTemp, Log, TEXT("Shutting Down Server"));
				FGenericPlatformMisc::RequestExit(false);
			}
		}
	});

	Socket->OnEvent(LobbyID + "/A", [this](const FString& Event, const TSharedPtr<FJsonValue>& Message)
	{
		UE_LOG(LogTemp, Log, TEXT("Received: %s"), *USIOJConvert::ToJsonString(Message));
	});

	Socket->OnEvent("join", [this](const FString& Event, const TSharedPtr<FJsonValue>& Message)
	{
		const auto JsonReader = TJsonReaderFactory<>::Create(USIOJConvert::ToJsonString(Message));
		if (TSharedPtr<FJsonObject> JsonObject; FJsonSerializer::Deserialize(JsonReader, JsonObject))
		{
			const auto JsonUsername = JsonObject->GetStringField("username");
			OnPlayerJoin.Broadcast(JsonUsername);
		}
	});

	Socket->OnEvent("leave", [this](const FString& Event, const TSharedPtr<FJsonValue>& Message)
	{
		const auto JsonReader = TJsonReaderFactory<>::Create(USIOJConvert::ToJsonString(Message));
		if (TSharedPtr<FJsonObject> JsonObject; FJsonSerializer::Deserialize(JsonReader, JsonObject))
		{
			const auto JsonUsername = JsonObject->GetStringField("username");
			OnPlayerLeave.Broadcast(JsonUsername);
		}
	});
}

void UCMGameInstance::Shutdown()
{
	Super::Shutdown();
	if (Socket.IsValid())
	{
		ISocketIOClientModule::Get().ReleaseNativePointer(Socket);
		Socket = nullptr;
	}
}

void UCMGameInstance::OnRequestToLoginComplete(FHttpRequestPtr Request, FHttpResponsePtr Response, bool bWasSuccessful)
{
	if (bWasSuccessful)
	{
		const auto JsonReader = TJsonReaderFactory<>::Create(Response->GetContentAsString());
		if (TSharedPtr<FJsonObject> JsonObject; FJsonSerializer::Deserialize(JsonReader, JsonObject))
		{
			const auto JsonToken = JsonObject->GetStringField("token");
			const auto JsonUsername = JsonObject->GetStringField("user");
			JwtToken = JsonToken;
			AccountUsername = JsonUsername;
			OnLoginComplete.Broadcast(true);
		}
		else
		{
			OnLoginComplete.Broadcast(false);
		}
	}
	else
	{
		OnLoginComplete.Broadcast(false);
	}
}

void UCMGameInstance::OnRequestToRegisterComplete(FHttpRequestPtr Request, FHttpResponsePtr Response, bool bWasSuccessful) const
{
	if (Response->GetResponseCode() == 201)
	{
		OnRegisterComplete.Broadcast(true);
	}
	else
	{
		OnRegisterComplete.Broadcast(false);
	}
}

void UCMGameInstance::OnRequestToLogoutComplete(FHttpRequestPtr Request, FHttpResponsePtr Response, bool bWasSuccessful)
{
	if (bWasSuccessful)
	{
		AccountUsername = "";
		JwtToken = "";
		OnLogoutComplete.Broadcast(true);
	}
	else
	{
		OnLogoutComplete.Broadcast(false);
	}
}

void UCMGameInstance::OnRequestToCreateLobbyComplete(FHttpRequestPtr Request, FHttpResponsePtr Response, bool bWasSuccessful)
{
	if (bWasSuccessful)
	{
		const auto JsonReader = TJsonReaderFactory<>::Create(Response->GetContentAsString());
		if (TSharedPtr<FJsonObject> JsonObject; FJsonSerializer::Deserialize(JsonReader, JsonObject))
		{
			const auto JsonLobbyID = JsonObject->GetStringField("lobbyID");
			const auto JsonPort = JsonObject->GetIntegerField("port");
			LobbyID = JsonLobbyID;
			ServerPort = JsonPort;
			OnCreateLobbyComplete.Broadcast(true);
			JoinLobby();
		}
		else
		{
			OnCreateLobbyComplete.Broadcast(false);
		}
	}
	else
	{
		OnCreateLobbyComplete.Broadcast(false);
	}
}

void UCMGameInstance::OnRequestToJoinLobbyComplete(FHttpRequestPtr Request, FHttpResponsePtr Response, bool bWasSuccessful)
{
	if (bWasSuccessful)
	{
		OnJoinLobbyComplete.Broadcast(true);
	}
	else
	{
		OnJoinLobbyComplete.Broadcast(false);
	}
}

void UCMGameInstance::OnRequestToLeaveLobbyComplete(FHttpRequestPtr Request, FHttpResponsePtr Response, bool bWasSuccessful)
{
	if (bWasSuccessful)
	{
		OnLeaveLobbyComplete.Broadcast(true);
	}
	else
	{
		OnLeaveLobbyComplete.Broadcast(false);
	}
}
